--A subparser.
local SubParser = {}
SubParser.__index = SubParser
package.loaded[...] = SubParser

--Constructor.
--
--@param Parser:Parser  Parent parser.
function SubParser.new(parser)
  --(1) arguments
  if not parser then error("parser expected.") end

  --(2) create
  return setmetatable({
    _ = {
      parser = parser,
      lexer = parser._.lexer
    }
  }, SubParser)
end

--Read next end of lines.
function SubParser:_skipEols()
  local lex = self._.lexer

  while true do
    local tok = lex:advance()

    if tok and tok:isEol() then
      lex:next()
    else
      break
    end
  end
end

--Read a path.
function SubParser:_nextPath(p)
  local lex = self._.lexer
  local path

  --(1) read
  local tok = lex:nextId()

  path = tok.value
  while lex:skipSymbol(".") do
    path = path .. "." .. lex:nextId().value
  end

  if p and path ~= p then
    self:_raise(string.format("'%s' expected.", p), tok)
  end

  --(2) return
  return path
end

--Throw a syntactic error.
--
--@param msg:string Message.
--@loc:object       Error location.
function SubParser:_raise(msg, loc)
  error(string.format(
    "%s (%s): %s",
    self._.lexer._.file,
    loc and string.format("%s, %s", loc.line, loc.col) or "end of file",
    msg
  ))
end
